unit graphs3d;

interface

uses Dos, Crt, Graph, bgi_chr;

type 
	vector = array[1..3] of Real;
	projection = array[1..2] of vector;

var 
	abs_x_center, abs_y_center : Integer;

procedure open_graph;
procedure close_graph;
procedure out_text_XY(ss : String; x, y : Integer; color : Word);
procedure put_pixel(x, y : Integer; color : Word);
procedure norm_line(x0, y0, x1, y1 : Integer; color : Word);
procedure compute_isometric_matrix(var P : projection);
procedure compute_dimetric_matrix(alpha : Real; var P : projection);
procedure compute_oblique_matrix(alpha : Real; var P : projection);
procedure project(const P : projection; const x, y, z : real; const  u0, v0 : Integer; var u, v : Integer);

implementation

var
	hold_color : Word;

procedure open_graph;

var
	GraphDriver, GraphMode : Integer;

begin
 GraphDriver := VGA;
 GraphMode := VGAHi;
 InitGraph(GraphDriver, GraphMode, '');
 SetBkColor(black);
 SetColor(white);
 SetTextStyle(SansSerifFont, HorizDir, 4);
 SetTextJustify(LeftText, BottomText);
 abs_x_center := (GetMaxX + 1) div 2;
 abs_y_center := (GetMaxY + 1) div 2;
end;

procedure close_graph;
var 
	gr: integer;
begin
 CloseGraph;
 gr := GraphResult;
 if  gr <> 0  then
 begin
	writeln('GraphResult = ', gr);
	readln;
	halt;
 end;
end;

procedure out_text_XY(ss : String; x, y : Integer; color : Word);
begin
 hold_color := getcolor;
 setcolor(color);
 OutTextXY(x + abs_x_center, abs_y_center - y, ss);
 setcolor(hold_color);
end;

procedure put_pixel(x, y : integer; color : word);
begin
 PutPixel(x + abs_x_center, abs_y_center - y, color);
end;

procedure norm_line(x0, y0, x1, y1 : Integer; color : Word);
begin
 hold_color := GetColor;
 SetColor(color);
 line(x0 + abs_x_center, abs_y_center - y0,
		x1 + abs_x_center, abs_y_center - y1);
 SetColor(hold_color);
end;

procedure compute_isometric_matrix(var P : projection);
begin
 P[1, 1] := -1.0 / Sqrt(2.0);
 P[1, 2] := -P[1, 1];
 P[1, 3] :=  0.0;
 P[2, 1] :=  -1.0 / Sqrt(6.0);
 P[2, 2] :=  P[2, 1];
 P[2, 3] := -2.0 * P[2, 1];
end;

procedure compute_dimetric_matrix(alpha : Real; var  P : projection);
var
	t : real;
begin
 alpha := pi * alpha / 180.0;
 P[1, 1] := -1.0 / sqrt(2.0);
 P[1, 2] := -P[1, 1];
 P[1, 3] := 0.0;
 t := sin(alpha) / cos(alpha);
 P[2, 1] := t * P[1, 1];
 P[2, 2] := P[2, 1];
 P[2, 3] := sqrt(1.0 - sqr(t));
end;

procedure compute_oblique_matrix(alpha : Real; var  P : projection);
begin
 alpha := pi * alpha / 180.0;
 P[1, 1] := - sin(alpha);
 P[1, 2] := 1.0;
 P[1, 3] := 0.0;
 P[2, 1] := -cos(alpha);
 P[2, 2] := 0.0;
 P[2, 3] := 1.0;
end;

procedure project(const  P : projection; const  x, y, z : real; const  u0, v0 : Integer; var  u, v : Integer);
begin
 u := u0 + round(P[1, 1] * x + P[1, 2] * y + P[1, 3] * z);
 v := v0 + round(P[2, 1] * x + P[2, 2] * y + P[2, 3] * z);
end;

begin
 if  RegisterBGIDriver(@EGAVGADriverProc) < 0  then
 begin
	ClrScr;
	WriteLn(GraphErrorMsg(GraphResult));
	Write('Press <Enter>; ');  readln;
	Halt;
 end;

 if  RegisterBGIFont(@SmallFontProc) < 0  then
 begin
	ClrScr;
	WriteLn(GraphErrorMsg(GraphResult));
	Write('Press <Enter>; ');  
	ReadLn;
	Halt;
 end;

 if  RegisterBGIFont(@SansSerifFontProc) < 0  then
 begin
	ClrScr;
	WriteLn(GraphErrorMsg(GraphResult));
	Write('Press <Enter>; ');  
	ReadLn;
	Halt;
 end;
end.
